@@ -2,13 +2,15 @@
 // SPDX-License-Identifier: BSD-2-Clause
 // Compares two files and returns whether they are the same once decoded.
 
+#include <cstdint>
 #include <iostream>
+#include <limits>
 #include <string>
 
 #include "aviftest_helpers.h"
 #include "avifutil.h"
 
-using libavif::testutil::AvifImagePtr;
+using avif::ImagePtr;
 
 int main(int argc, char** argv) {
   if (argc != 4 && argc != 5) {
@@ -16,9 +18,8 @@ int main(int argc, char** argv) {
               << " file1 file2 ignore_alpha_flag [psnr_threshold]" << std::endl;
     return 2;
   }
-  AvifImagePtr decoded[2] = {
-      AvifImagePtr(avifImageCreateEmpty(), avifImageDestroy),
-      AvifImagePtr(avifImageCreateEmpty(), avifImageDestroy)};
+  ImagePtr decoded[2] = {ImagePtr(avifImageCreateEmpty()),
+                         ImagePtr(avifImageCreateEmpty())};
   if (!decoded[0] || !decoded[1]) {
     std::cerr << "Cannot create AVIF images." << std::endl;
     return 2;
@@ -35,6 +36,9 @@ int main(int argc, char** argv) {
                       /*ignoreColorProfile==*/AVIF_FALSE,
                       /*ignoreExif=*/AVIF_FALSE,
                       /*ignoreXMP=*/AVIF_FALSE, /*allowChangingCicp=*/AVIF_TRUE,
+                      // TODO(maryla): also compare gain maps.
+                      /*ignoreGainMap=*/AVIF_TRUE,
+                      /*imageSizeLimit=*/std::numeric_limits<uint32_t>::max(),
                       decoded[i].get(), &depth[i], nullptr,
                       nullptr) == AVIF_APP_FILE_FORMAT_UNKNOWN) {
       std::cerr << "Image " << argv[i + 1] << " cannot be read." << std::endl;
@@ -51,8 +55,8 @@ int main(int argc, char** argv) {
   bool ignore_alpha = std::stoi(argv[3]) != 0;
 
   if (argc == 4) {
-    if (!libavif::testutil::AreImagesEqual(*decoded[0], *decoded[1],
-                                           ignore_alpha)) {
+    if (!avif::testutil::AreImagesEqual(*decoded[0], *decoded[1],
+                                        ignore_alpha)) {
       std::cerr << "Images " << argv[1] << " and " << argv[2]
                 << " are different." << std::endl;
       return 1;
@@ -60,8 +64,7 @@ int main(int argc, char** argv) {
     std::cout << "Images " << argv[1] << " and " << argv[2] << " are identical."
               << std::endl;
   } else {
-    auto psnr =
-        libavif::testutil::GetPsnr(*decoded[0], *decoded[1], ignore_alpha);
+    auto psnr = avif::testutil::GetPsnr(*decoded[0], *decoded[1], ignore_alpha);
     if (psnr < std::stod(argv[4])) {
       std::cerr << "PSNR: " << psnr << ", images " << argv[1] << " and "
                 << argv[2] << " are not similar." << std::endl;
