@@ -145,7 +145,7 @@ void FillImagePlain(avifImage* image, const uint32_t yuva[4]) {
   }
 }
 
-void FillImageGradient(avifImage* image) {
+void FillImageGradient(avifImage* image, int offset) {
   for (avifChannelIndex c :
        {AVIF_CHAN_Y, AVIF_CHAN_U, AVIF_CHAN_V, AVIF_CHAN_A}) {
     const uint32_t limitedRangeMin =
@@ -158,16 +158,17 @@ void FillImageGradient(avifImage* image) {
     const uint32_t plane_height = avifImagePlaneHeight(image, c);
     uint8_t* row = avifImagePlane(image, c);
     const uint32_t row_bytes = avifImagePlaneRowBytes(image, c);
+    const uint32_t max_xy_sum = plane_width + plane_height - 2;
     for (uint32_t y = 0; y < plane_height; ++y) {
       for (uint32_t x = 0; x < plane_width; ++x) {
-        uint32_t value;
+        uint32_t value = (x + y + offset) % (max_xy_sum + 1);
         if (image->yuvRange == AVIF_RANGE_FULL || c == AVIF_CHAN_A) {
-          value = (x + y) * ((1u << image->depth) - 1u) /
-                  std::max(1u, plane_width + plane_height - 2);
+          value =
+              value * ((1u << image->depth) - 1u) / std::max(1u, max_xy_sum);
         } else {
-          value = limitedRangeMin +
-                  (x + y) * (limitedRangeMax - limitedRangeMin) /
-                      std::max(1u, plane_width + plane_height - 2);
+          value = limitedRangeMin + value *
+                                        (limitedRangeMax - limitedRangeMin) /
+                                        std::max(1u, max_xy_sum);
         }
         if (avifImageUsesU16(image)) {
           reinterpret_cast<uint16_t*>(row)[x] = static_cast<uint16_t>(value);
@@ -509,9 +510,7 @@ AvifRwData Encode(const avifImage* image, int speed, int quality) {
   encoder->speed = speed;
   encoder->quality = quality;
   encoder->qualityAlpha = quality;
-#if defined(AVIF_ENABLE_EXPERIMENTAL_GAIN_MAP)
   encoder->qualityGainMap = quality;
-#endif
   testutil::AvifRwData bytes;
   if (avifEncoderWrite(encoder.get(), image, &bytes) != AVIF_RESULT_OK) {
     return {};
