@@ -16,43 +16,24 @@
 #
 # tests for command lines (lossless)
 
-# Very verbose but useful for debugging.
-set -ex
-
-if [[ "$#" -ge 1 ]]; then
-  # eval so that the passed in directory can contain variables.
-  BINARY_DIR="$(eval echo "$1")"
-else
-  # Assume "tests" is the current directory.
-  BINARY_DIR="$(pwd)/.."
-fi
-if [[ "$#" -ge 2 ]]; then
-  TESTDATA_DIR="$(eval echo "$2")"
-else
-  TESTDATA_DIR="$(pwd)/data"
-fi
-if [[ "$#" -ge 3 ]]; then
-  TMP_DIR="$(eval echo "$3")"
-else
-  TMP_DIR="$(mktemp -d)"
-fi
-
-AVIFENC="${BINARY_DIR}/avifenc"
-AVIFDEC="${BINARY_DIR}/avifdec"
-ARE_IMAGES_EQUAL="${BINARY_DIR}/tests/are_images_equal"
+source $(dirname "$0")/cmd_test_common.sh
 
 # Input file paths.
 INPUT_PNG="${TESTDATA_DIR}/paris_icc_exif_xmp.png"
 INPUT_GRAY_PNG="${TESTDATA_DIR}/kodim03_grayscale_gamma1.6.png"
+INPUT_Y4M="${TESTDATA_DIR}/kodim03_yuv420_8bpc.y4m"
 # Output file names.
 ENCODED_FILE="avif_test_cmd_lossless_encoded.avif"
 DECODED_FILE="avif_test_cmd_lossless_decoded.png"
 DECODED_FILE_LOSSLESS="avif_test_cmd_lossless_decoded_lossless.png"
+DECODED_FILE_Y4M="avif_test_cmd_lossless_decoded_lossless.y4m"
+OUT_MSG="avif_test_cmd_lossless_out_msg.txt"
 
 # Cleanup
 cleanup() {
   pushd ${TMP_DIR}
-    rm -- "${ENCODED_FILE}" "${DECODED_FILE}" "${DECODED_FILE_LOSSLESS}"
+    rm -f -- "${ENCODED_FILE}" "${DECODED_FILE}" "${DECODED_FILE_LOSSLESS}" \
+             "${DECODED_FILE_Y4M}" "${OUT_MSG}"
   popd
 }
 trap cleanup EXIT
@@ -78,11 +59,31 @@ pushd ${TMP_DIR}
   "${AVIFDEC}" "${ENCODED_FILE}" "${DECODED_FILE_LOSSLESS}"
   "${ARE_IMAGES_EQUAL}" "${INPUT_PNG}" "${DECODED_FILE_LOSSLESS}" 0
 
+  echo "Testing YCgCo-Re lossless"
+  "${AVIFENC}" -s 10 -l --cicp 2/2/16 "${INPUT_PNG}" -o "${ENCODED_FILE}"
+  "${AVIFDEC}" "${ENCODED_FILE}" "${DECODED_FILE_LOSSLESS}"
+  "${ARE_IMAGES_EQUAL}" "${INPUT_PNG}" "${DECODED_FILE_LOSSLESS}" 0
+
+  echo "Testing YCgCo-Ro lossless fails because the input bit depth is even"
+  "${AVIFENC}" -s 10 -l --cicp 2/2/17 "${INPUT_PNG}" -o "${ENCODED_FILE}" && exit 1
+
   # 400 test.
   echo "Testing 400 lossless"
   "${AVIFENC}" -y 400 -s 10 -l "${INPUT_GRAY_PNG}" -o "${ENCODED_FILE}"
   "${AVIFDEC}" "${ENCODED_FILE}" "${DECODED_FILE_LOSSLESS}"
   "${ARE_IMAGES_EQUAL}" "${INPUT_GRAY_PNG}" "${DECODED_FILE_LOSSLESS}" 0
+
+  # SVT-AV1 supports 4:2:0 lossless starting with version 3.0.0.
+  # SVT-AV1 does not support 4:4:4 so far, so use a 4:2:0 y4m as test input.
+  if "${AVIFENC}" --version | grep 'svt \['; then
+    # Depending on the version, either succeeds or prints an error message.
+    if "${AVIFENC}" -c svt --lossless --cicp 2/2/16 -s 9 "${INPUT_Y4M}" "${ENCODED_FILE}" 2> "${OUT_MSG}"; then
+      "${AVIFDEC}" "${ENCODED_FILE}" "${DECODED_FILE_Y4M}"
+      "${ARE_IMAGES_EQUAL}" "${INPUT_Y4M}" "${DECODED_FILE_Y4M}" 0
+    else
+      grep "lossless support was added in version 3" "${OUT_MSG}"
+    fi
+  fi
 popd
 
 exit 0
